require 'fileutils'
require 'thor'

module Vara
  class Cli < Thor
    def self.exit_on_failure?
      true
    end

    desc 'build-metadata PRODUCT_DIR', 'Compose metadata from metadata_parts'
    option 'cycle', default: 'alpha'
    option 'final', type: :boolean, desc: 'When true, $PRERELEASE_VERSION$ is removed instead of substituted'
    option 'version', type: :string, desc: 'When given, use the version as specified'
    option 'external_releases', desc: 'Comma delimited paths to release metadata files to use when generating the .pivotal'
    def build_metadata(product_dir)
      require 'vara/prerelease_versioner'
      require 'vara/static_versioner'
      require 'vara/product_metadata_processor'
      versioner = Vara::PrereleaseVersioner.new(File.expand_path(product_dir), options.fetch('cycle'))
      versioner.override_prerelease_version!('') if options['final']
      if options['version'] && !options['version'].empty?
        versioner = Vara::StaticVersioner.new(File.expand_path(product_dir), options.fetch('version'))
      end
      metadata_path = Vara::ProductMetadataProcessor.new(
        File.expand_path(product_dir),
        versioner,
        external_release_paths: options.fetch('external_releases', '').split(',')
      ).process
      say("#{metadata_path} has been created")
      metadata_path
    end

    desc 'build-migrations PRODUCT_DIR', 'Compose content_migrations from content_migrations_parts'
    option 'cycle', default: 'alpha'
    option 'final', type: :boolean, desc: 'When true, $PRERELEASE_VERSION$ is removed instead of substituted'
    option 'version', type: :string, desc: 'When given, use the version as specified'
    def build_migrations(product_dir)
      require 'vara/prerelease_versioner'
      require 'vara/static_versioner'
      require 'vara/content_migrations_processor'
      versioner = Vara::PrereleaseVersioner.new(File.expand_path(product_dir), options.fetch('cycle'))
      versioner.override_prerelease_version!('') if options['final']
      if options['version'] && !options['version'].empty?
        versioner = Vara::StaticVersioner.new(File.expand_path(product_dir), options.fetch('version'))
      end
      content_migrations_path = Vara::ContentMigrationsProcessor.new(File.expand_path(product_dir), versioner).process
      say("#{content_migrations_path} has been created") unless content_migrations_path.nil?
      content_migrations_path
    end

    desc 'lint PRODUCT_METADATA', 'Lints the product metadata file'
    def lint(product_metadata)
      require 'vara/linter'
      linter = Vara::Linter.build(File.expand_path(product_metadata))
      linter.lint!
    end

    desc 'download-artifacts PRODUCT_METADATA', 'Download the artifacts from the product metadata file'
    def download_artifacts(product_metadata)
      require 'vara/product_resource_downloader'
      product_resource_downloader = Vara::ProductResourceDownloader.build(File.expand_path(product_metadata))
      product_resource_downloader.download
    end

    desc 'zip-pivotal PRODUCT_DIR', 'Zips up the .pivotal product given that all artifacts are synced locally'
    def zip_pivotal(product_dir)
      require 'vara/product'
      product = Vara::Product.new(File.expand_path(product_dir), options)
      product.build
      say("file #{product.path} has been created")
      say("file #{product.bom_path} has been created")
      say("file #{product.md5_path} has been created")
    end

    desc 'build-pivotal PRODUCT_DIR', 'Generate a .pivotal from a vara-flavored product repository'
    option 'final', type: :boolean, desc: 'When true, $PRERELEASE_VERSION$ is removed instead of substituted'
    option 'exclude_releases', type: :boolean, desc: 'When true, the empty files will be included instead of actual release binaries'
    option 'version', type: :string, desc: 'When given, use the version as specified'
    option 'external_releases', desc: 'Comma delimited paths to release metadata files to use when generating the .pivotal'
    option 'cycle', default: 'alpha'
    def build_pivotal(product_dir)
      metadata_path = build_metadata(product_dir)
      build_migrations(product_dir)
      lint(metadata_path)
      download_artifacts(metadata_path) unless options[:exclude_releases]
      zip_pivotal(product_dir)
    end

    desc 'update-product-version PIVOTAL_FILE_PATH VERSION', 'Update the version of a pivotal file'
    def update_product_version(pivotal_file_path, version)
      require 'vara/content_migrations_processor'
      require 'vara/product'
      require 'vara/static_versioner'
      pivotal_file_path = File.expand_path(pivotal_file_path)
      Dir.mktmpdir do |temp_dir|
        FileUtils.cd(temp_dir) do
          `unzip #{pivotal_file_path} -d .`
          output_dir = File.dirname(pivotal_file_path)

          versioner = Vara::StaticVersioner.new(temp_dir, version)
          Vara::ContentMigrationsProcessor.new(temp_dir, versioner).update_product_version_migrations

          product = Vara::Product.new(temp_dir)
          product.update_product_version(version)
          product.build
          product.paths.each do |path|
            FileUtils.mv(path, output_dir)
          end
        end
      end
    end
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
