require 'yaml'
require 'vara/log'

module Vara
  class ContentMigrationsProcessor
    include Vara::Loggable

    # @param product_dir [String] path to product directory root
    # @param versioner [Vara::PrereleaseVersioner,Vara::StaticVersioner]
    def initialize(product_dir, versioner)
      @product_dir = product_dir
      @versioner = versioner
    end

    # Composes content_migrations_parts/base.yml and content_migrations_parts/migrations/*.yml
    # into content_migrations/migrations.yml, expanding placeholders as necessary.
    # @return [String] path to the generated content migration file
    def process
      return unless File.exist?(parts_dir)
      content_migrations = load_content_migrations
      processed = versioner.update_content_migrations(content_migrations)
      save_content_migrations(processed)
    end

    def update_product_version_migrations
      return unless File.exists? migrations_file_path
      migrations = YAML.load_file(migrations_file_path)
      versioner.update_content_migrations(migrations)
      save_content_migrations(migrations)
    end

    private

    attr_reader :product_dir, :versioner

    def load_content_migrations
      base_path = File.join(parts_dir, 'base.yml')
      content_migrations = YAML.load_file(base_path) || {}
      log.info("Composing content migrations: loaded #{base_path}")
      Dir.glob(File.join(parts_dir, 'migrations', '*.yml')).each do |migration_path|
        migration = YAML.load_file(migration_path)
        log.info("Composing content migrations: loaded #{migration_path}")
        content_migrations['migrations'] ||= []
        content_migrations['migrations'] << migration
      end

      content_migrations
    end

    def parts_dir
      File.join(product_dir, 'content_migrations_parts')
    end

    def save_content_migrations(processed_hash)
      out_path = migrations_file_path

      File.open(out_path, 'w') do |out|
        YAML.dump(processed_hash, out)
      end

      out_path
    end

    def migrations_file_path
      File.join(product_dir, 'content_migrations', 'migrations.yml')
    end
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
