require 'yaml'
require 'digest/md5'
require 'vara/product_metadata'
require 'vara/git_inspector'
require 'vara/log'
require 'vara/version'

module Vara
  class Materials
    include Vara::Loggable

    # @param materials_path [String] Path to a materials YAML file
    # @return [Vara::Materials] A Materials object based on the content of the file at materials_path
    def self.from_file(materials_path)
      new(YAML.load_file(materials_path))
    end

    # @param product_metadata_path [String] Path to a product's metadata file.
    #   Expected to be in metadata/ folder of a product repo.
    # @param product_path [String] Path to a built .pivotal file on disk
    # @return [Vara::Materials]
    def self.build(product_metadata_path, product_path)
      product_metadata_dir = File.dirname(File.expand_path(product_metadata_path))
      repo_dir = File.expand_path(File.join(product_metadata_dir, '..'))
      repo_name = File.basename(repo_dir)

      product_metadata = ProductMetadata.from_file(product_metadata_path)

      repo_revision = if GitInspector.git_repo?(repo_dir)
                        GitInspector.new(repo_dir).current_revision
                      else
                        log.warn('Creating Materials from a non-git repository')
                        'not_a_git_repository'
                      end

      materials_hash = {
        'filename' => File.basename(product_path),
        'md5' => Digest::MD5.file(product_path).hexdigest,
        'contents' => {
          'releases' => product_metadata.releases_metadata.map(&:basename),
          'compiled_packages' => []
        },
        'build_info' => {
          'product_repository' => repo_name,
          'product_revision' => repo_revision,
          'vara_version' => Vara::VERSION
        }
      }

      if product_metadata.explicit_stemcell?
        materials_hash['contents']['stemcells'] = [product_metadata.stemcell_metadata.basename]

        if product_metadata.has_compiled_packages?
          materials_hash['contents']['compiled_packages'] << product_metadata.compiled_packages_metadata.basename
        end
      else
        materials_hash['contents']['stemcell_criteria'] = product_metadata.stemcell_criteria
      end

      new(materials_hash)
    end

    # @param materials_hash [Hash]
    def initialize(materials_hash)
      @hash = materials_hash
    end

    def save_to(file_path)
      File.open(file_path, 'w') { |file| YAML.dump(hash, file) }
    end

    # @return [String] The filename of the .pivotal file
    def filename
      hash.fetch('filename')
    end

    # @return [String] The listed md5 of the .pivotal file
    def md5
      hash.fetch('md5')
    end

    # @return [<String>] The listed releases contained in the .pivotal file
    def releases
      contents.fetch('releases')
    end

    # @return [<String>] The listed stemcells contained in the .pivotal file
    def stemcells
      contents['stemcells']
    end

    def stemcell_criteria
      contents['stemcell_criteria']
    end

    # @return [<String>] The listed compiled packages contained in the .pivotal file
    def compiled_packages
      contents.fetch('compiled_packages')
    end

    # @return [String] The name of the product repository
    def product_repository
      build_info.fetch('product_repository')
    end

    # @return [String] The git revision of the product repository used to build the .pivotal
    def product_revision
      build_info.fetch('product_revision')
    end

    # @return [String] the version of vara used to build the described .pivotal (will be >= 0.7.1 or 'unknown')
    def vara_version
      build_info.fetch('vara_version', 'unknown')
    end

    private

    attr_reader :hash

    # @return [Hash]
    def contents
      hash.fetch('contents')
    end

    # @return [Hash]
    def build_info
      hash.fetch('build_info')
    end
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
