module Vara
  module Metadata
    # "Struct" representing the information about the compiled_packages field in metadata.yml
    # @example entry from metadata.yml
    #  compiled_package:
    #    name: cf
    #    file: cf-170-bosh-vsphere-esxi-ubuntu-2366.tgz
    #    version: "170"
    #    md5: 048f9e546bb2bb490f44cd9ea074b22a
    #    url: https://s3-us-west-1.amazonaws.com/releng-artifacts/cf-170-build-99/cf-170-bosh-vsphere-esxi-ubuntu-2366.tgz
    class CompiledPackages
      # The name field of the YAML. Typically the name of the contained release.
      # @return [String]
      attr_reader :name

      # The version field of the YAML.  Typically the version of the contained release.
      # @return [String]
      attr_reader :version

      # The filename of the compiled package
      # @return [String]
      attr_reader :basename

      # Checksum of the compiled_packages tarball
      # @return [String]
      attr_reader :md5

      # SHA1 Checksum is not supported for compiled_packages
      # @return nil
      attr_reader :sha1

      # Infers the metadata given a compiled packages file on disk
      # @param path_to_compiled_packages [String] The path to the compiled packages file on disk
      # @return [Vara::CompiledPackagesMetadata]
      def self.from_file(path_to_compiled_packages)
        md5      = Digest::MD5.file(path_to_compiled_packages).hexdigest
        basename = File.basename(path_to_compiled_packages)

        name, rest = basename.split('-', 2)
        version    = rest.gsub(/-bosh.*$/, '')

        new(name, version, basename, md5)
      end

      # @param name [String]
      # @param version [String]
      # @param file [String]
      # @param md5 [String]
      # @param url [String]
      def initialize(name, version, file, md5, url = nil)
        @name     = name
        @version  = version
        @basename = file
        @md5      = md5
        @url      = url
      end

      # The blobstore URL for the compiled_package.
      # @raise [RuntimeError] if the URL is not set.
      # @return [String]
      def url
        @url || raise("URL unknown for compiled_packages #{name}")
      end

      # @return [nil] the interface for downloader requires this method
      def aws
        nil
      end

      # If the URL is set, a markdown-style URL with the basename attribute.
      # Otherwise, just the basename.
      # @return [String]
      def to_s
        @url ? "[#{basename}](#{@url})" : basename
      end
    end
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
