require 'ostruct'

module Vara
  module Metadata
    # "Struct" representing the information about a single release in metadata.yml's releases array
    # @example entry from metadata.yml
    #   releases:
    #     - file: cf-170.tgz
    #       name: cf
    #       version: '170'
    #       md5: 8a0700de688b4fdd270b41c4a4de9d0e
    #       url: https://releng-artifacts.s3.amazonaws.com/cf-170.tgz
    class Release
      # @return [String] Name of the release
      attr_reader :name

      # @return [String] Version of the release
      attr_reader :version

      # @return [String] The name of the release file
      attr_reader :basename

      # @return [String] The md5 of the release file's content
      attr_reader :md5

      # @return [String] The sha1 of the release file's content
      attr_reader :sha1

      # @return [OpenStruct] The AWS Configuration
      attr_reader :aws

      # Infers the metadata given a release file on disk
      # @param release_tarball_path [String] The path to the release file on disk
      # @return [Vara::ReleaseMetadata]
      def self.from_file(release_tarball_path)
        md5      = Digest::MD5.file(release_tarball_path).hexdigest
        sha1     = Digest::SHA1.file(release_tarball_path).hexdigest
        basename = File.basename(release_tarball_path)

        release_parts = basename.gsub(/\.tgz$/, '')
        name, version = release_parts.split('-', 2)

        new(name: name, version: version, file: basename, md5: md5, sha1: sha1)
      end

      # @param name [String]
      # @param version [String]
      # @param file [String]
      # @param md5 [String]
      # @param url [String]
      def initialize(name:, version:, file:, md5:, sha1:, url: nil, aws: nil)
        @name     = name
        @version  = version
        @basename = file
        @md5      = md5
        @sha1     = sha1
        @url      = url
        @aws = OpenStruct.new(aws) if aws
      end

      def url
        if @url
          @url
        elsif @aws
          "#{aws.bucket_name}/#{aws.filename}"
        end
      end

      # If the URL is set, a markdown-style URL with the basename attribute.
      # Otherwise, just the basename.
      # @return [String]
      def to_s
        @url ? "[#{basename}](#{@url})" : basename
      end
    end
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
