module Vara
  module Metadata
    # "Struct" representing the information about the stemcell field in metadata.yml
    # @note Unlike Vara::ReleaseMetadata and Vara::CompiledPackagesMetadata, this class does not have a settable URL
    #   because there is an inferred URL from the global BOSH stemcell blobstore.
    # @example entry from metadata.yml
    #   stemcell:
    #     name: bosh-vsphere-esxi-ubuntu
    #     version: '2366'
    #     file: bosh-stemcell-2366-vsphere-esxi-ubuntu.tgz
    #     md5: 0fbdcd100f716d127f821b1b4335135a
    class Stemcell
      # @return [String] the name of the stemcell.
      attr_reader :name

      # @return [String] the version of the stemcell.
      attr_reader :version

      # @return [String] the filename of the stemcell
      attr_reader :basename

      # @return [String] md5 checksum of the stemcell
      attr_reader :md5

      # @return [String] sha1 checksum of the stemcell
      attr_reader :sha1

      # Infers the metadata given a stemcell file on disk
      # @param path_to_stemcell [String] The path to the stemcell file on disk
      # @return [Vara::StemcellMetadata]
      def self.from_file(path_to_stemcell)
        md5      = Digest::MD5.file(path_to_stemcell).hexdigest
        sha1     = Digest::SHA1.file(path_to_stemcell).hexdigest
        basename = File.basename(path_to_stemcell)

        stemcell_parts = basename.gsub(/^bosh-stemcell-/, '').gsub(/\.tgz$/, '')

        raw_version, iaas, hypervisor, os = stemcell_parts.split('-')
        version                           = raw_version.tr('_', '.')

        name = ['bosh', iaas, hypervisor, os].join('-')

        new(name: name, version: version, file: basename, md5: md5, sha1: sha1)
      end

      # @param name [String]
      # @param version [String]
      # @param file [String]
      # @param md5 [String]
      def initialize(name:, version:, file:, md5:, sha1:)
        @name     = name
        @version  = version
        @basename = file
        @md5      = md5
        @sha1     = sha1
      end

      # @return [String] the inferred blobstore URL of the stemcell, assuming it is a BOSH public stemcell
      def url
        iaas = basename.gsub(/.*\d-(\w+)-.*/, '\1')
        "http://bosh-jenkins-artifacts.cf-app.com/bosh-stemcell/#{iaas}/#{basename}"
      end

      # @return [nil] the interface for downloader requires this method
      def aws
        nil
      end

      # @return [String] a markdown-style URL with the basename attribute.
      def to_s
        "[#{basename}](#{url})"
      end
    end
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
