require 'vara/git_inspector'

module Vara
  class PrereleaseVersioner
    PRERELEASE_VERSION_PLACEHOLDER = '$PRERELEASE_VERSION$'.freeze
    PRODUCT_VERSION_KEY = 'product_version'.freeze
    TO_VERSION_KEY = 'to_version'.freeze
    MIGRATIONS_KEY = 'migrations'.freeze
    MIGRATIONS_RULES_KEY = 'rules'.freeze
    MIGRATIONS_RULES_TO_KEY = 'to'.freeze

    # @param product_directory_path [String]
    # @param cycle [String] cycle (e.g. alpha, beta, etc.) to use in expanded prerelease version value
    def initialize(product_directory_path, cycle)
      @product_directory_path = product_directory_path
      @cycle = cycle
    end

    # @param new_prerelease_version [String] explicit value to override the prerelease version. Use with caution.
    # @return [void]
    def override_prerelease_version!(new_prerelease_version)
      @prerelease_version = new_prerelease_version
    end

    # @param metadata_hash [Hash] Hash contents of full product metadata
    # @return [Hash] product metadata with product version value updated if necessary
    def update_metadata(metadata_hash)
      return metadata_hash unless metadata_hash.fetch(PRODUCT_VERSION_KEY).include?(PRERELEASE_VERSION_PLACEHOLDER)
      metadata_hash[PRODUCT_VERSION_KEY].gsub!(PRERELEASE_VERSION_PLACEHOLDER, prerelease_version)
      metadata_hash
    end

    # @param migrations_hash [Hash] Hash contents of full content migrations
    # @return [Hash] content migrations with product version value updated if necessary
    def update_content_migrations(migrations_hash)
      migrations_hash[TO_VERSION_KEY].gsub!(PRERELEASE_VERSION_PLACEHOLDER, prerelease_version) if migrations_hash.key?(TO_VERSION_KEY)
      migrations_hash.fetch(MIGRATIONS_KEY, []).each do |migration|
        migration.fetch(MIGRATIONS_RULES_KEY, []).each do |rule|
          next unless updates_product_version?(rule)
          rule[MIGRATIONS_RULES_TO_KEY].gsub!(PRERELEASE_VERSION_PLACEHOLDER, prerelease_version)
        end
      end
      migrations_hash
    end

    private

    attr_reader :product_directory_path, :cycle

    def updates_product_version?(rule_hash)
      rule_hash['type'] == 'update' && rule_hash['selector'] == 'product_version'
    end

    def prerelease_version
      @prerelease_version ||= ".#{cycle}.#{commit_number}.#{short_sha}"
    end

    def commit_number
      git_inspector.commit_count
    end

    def short_sha
      git_inspector.short_sha
    end

    def git_inspector
      @git_inspector ||= GitInspector.new(product_directory_path)
    end
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
