require 'vara/product_metadata'
require 'vara/product_resource_downloader'
require 'vara/product_artifact_zipper'
require 'vara/product_contents'
require 'vara/md5_creator'
require 'vara/materials'

module Vara
  class Product
    METADATA_DIR       = 'metadata'.freeze
    METADATA_PARTS_DIR = 'metadata_parts'.freeze

    attr_reader :base_dir

    def initialize(base_dir, options = {})
      @base_dir = base_dir
      @options = options
    end

    def metadata_file
      @metadata_file ||= find_metadata_file || raise('No metadata file found.')
    end

    def build
      if metadata.explicit_stemcell? && metadata.stemcell_criteria?
        raise 'binaries.yml includes both stemcell and stemcell criteria keys'
      end

      ProductArtifactZipper.new(path, ProductContents.from_metadata_path(metadata_file), @options).zip!

      MD5Creator.md5(path)

      Materials.build(metadata_file, path).save_to(bom_path)

      path
    end

    def path
      File.join(base_dir, "#{filename_components.join('-')}.pivotal")
    end

    def bom_path
      "#{path}.yml"
    end

    def md5_path
      "#{path}.md5"
    end

    def paths
      [path, bom_path, md5_path]
    end

    def stemcell_version
      metadata.stemcell_metadata.version
    end

    def metadata_ref
      `cd #{base_dir} && git log -1 --format=format:%h`
    end

    def releases
      metadata.releases_metadata.map do |release_metadata|
        {
          'name'    => release_metadata['name'],
          'version' => release_metadata['version']
        }
      end
    end

    def update_product_version(new_version)
      metadata_hash = metadata.hash
      metadata_hash['product_version'] = new_version
      File.open(metadata_file, 'w') do |out|
        YAML.dump(metadata_hash, out)
      end
    end

    private

    def metadata
      ProductMetadata.from_file(metadata_file)
    end

    def filename_components
      [metadata.name, metadata.product_version]
    end

    def find_metadata_file
      files = Dir.glob(File.join(base_dir, METADATA_DIR, '*.yml')).sort

      case files.size
      when 0 then
        nil
      when 1 then
        files.first
      else
        raise("Found: #{files.map { |f| File.basename(f) }}. Vara supports one .yml file under metadata/")
      end
    end
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
