require 'vara/product_metadata'

module Vara
  class ProductContents
    # @param metadata_path [String] The path on disk to the metadata YAML file
    # @return [Vara::ProductContents]
    def self.from_metadata_path(metadata_path)
      new(metadata_path, ProductMetadata.from_file(metadata_path))
    end

    attr_reader :metadata_path

    # @param metadata_path [String] The path on disk to the metadata YAML file
    # @param product_metadata [Vara::ProductMetadata] The Product Metadata as represented in the metadata_path file
    def initialize(metadata_path, product_metadata)
      @metadata_path    = File.expand_path(metadata_path).freeze
      @product_metadata = product_metadata
    end

    # @return [String] The path to the content migration file on disk
    def content_migrations_path
      paths = Dir.glob(File.expand_path(File.join(root_path, 'content_migrations', '*.yml')))
      raise "Exactly one content_migrations file is required in #{root_path}/content_migrations" if paths.size != 1
      paths.first
    end

    def content_migrations_exist?
      File.exist?(File.expand_path(File.join(root_path, 'content_migrations')))
    end

    def migrations_path
      File.expand_path(File.join(root_path, 'migrations'))
    end

    def migrations_exist?
      File.exist?(File.expand_path(File.join(root_path, 'migrations')))
    end

    # @return [<String>] The paths to the release tarballs on disk
    def release_paths
      product_metadata.releases_metadata.map { |release_metadata| File.join(root_path, 'releases', release_metadata.basename) }
    end

    # @return [String] The path to the stemcell file on disk
    def stemcell_path
      File.join(root_path, 'stemcells', product_metadata.stemcell_file)
    end

    # @return [String] The path to the compiled packages file on disk
    def compiled_packages_path
      File.join(root_path, 'compiled_packages', product_metadata.compiled_packages_file)
    end

    # @return [Boolean] Whether the product metadata has a compiled packages field
    def has_compiled_packages?
      product_metadata.has_compiled_packages?
    end

    private

    def root_path
      File.expand_path(File.join(metadata_path, '..', '..'))
    end

    attr_reader :product_metadata
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
