require 'yaml'
require 'vara/metadata/stemcell'
require 'vara/metadata/release'
require 'vara/metadata/compiled_packages'

module Vara
  # "Struct" representing the information in metadata/<product_name>.yml
  class ProductMetadata

    attr_reader :hash

    # Build a ProductMetadata from a metadata.yml on disk
    # @param metadata_path Path to the product metadata file on disk
    # @return [Vara::ProductMetadata]
    def self.from_file(metadata_path)
      new(YAML.load_file(metadata_path))
    end

    # @param hash [Hash] the hash representing the product metadata
    def initialize(hash)
      @hash = hash
    end

    def explicit_stemcell?
      hash.key? 'stemcell'
    end

    def stemcell_criteria
      return nil if explicit_stemcell?
      hash.fetch('stemcell_criteria')
    end

    def stemcell_criteria?
      hash.key? 'stemcell_criteria'
    end

    # @return [Vara::Metadata::Stemcell] the stemcell metadata contained in this ProductMetadata
    def stemcell_metadata
      stemcell = hash.fetch('stemcell')
      Metadata::Stemcell.new(
        name: stemcell.fetch('name'),
        version: stemcell.fetch('version'),
        file: stemcell.fetch('file'),
        md5: stemcell.fetch('md5', nil),
        sha1: stemcell.fetch('sha1', nil)
      )
    end

    # @return [<Vara::Metadata::Release>] the releases metadata contained in this ProductMetadata
    def releases_metadata
      releases = hash.fetch('releases')
      releases.map { |release| release_metadata(release) }
    end

    # @raise [KeyNotFoundError] if this ProductMetadata does not contain compiled packages
    # @return [Vara::Metadata::CompiledPackages] the compiled packages metadata contained in this ProductMetadata
    def compiled_packages_metadata
      c = hash.fetch('compiled_package')
      Metadata::CompiledPackages.new(
        c.fetch('name'), c.fetch('version'), c.fetch('file'), c.fetch('md5'), c.fetch('url', nil)
      )
    end

    # @return [String] the filename of the stemcell
    def stemcell_file
      stemcell_metadata.basename
    end

    # @return [String] the name of the product
    def name
      hash.fetch('name')
    end

    # @return [String] the version of the product
    def product_version
      hash.fetch('product_version')
    end

    # @return [String] the filename of the compiled packages
    def compiled_packages_file
      compiled_packages_metadata.basename
    end

    # @return [Boolean] whether the compiled packages field is present
    def has_compiled_packages?
      hash.key?('compiled_package')
    end

    private

    # @return [Vara::Metadata::Release]
    def release_metadata(release_hash)
      release = release_hash
      Metadata::Release.new(
        name: release.fetch('name'),
        version: release.fetch('version'),
        file: release.fetch('file'),
        md5: release.fetch('md5', nil),
        sha1: release.fetch('sha1', nil),
        url: release.fetch('url', nil),
        aws: release.fetch('aws', nil)
      )
    end
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
