module Vara
  class VersionError < RuntimeError; end

  class StaticVersioner
    PRODUCT_VERSION_KEY = 'product_version'.freeze
    TO_VERSION_KEY = 'to_version'.freeze
    MIGRATIONS_KEY = 'migrations'.freeze
    MIGRATIONS_RULES_KEY = 'rules'.freeze
    MIGRATIONS_RULES_TO_KEY = 'to'.freeze

    # @param product_directory_path [String]
    # @param version [String] the product version
    def initialize(product_directory_path, version)
      begin
        Gem::Version.create(version)
      rescue
        raise Vara::VersionError, "Invalid version: #{version}"
      end

      @product_directory_path = product_directory_path
      @version = version
    end

    # @param metadata_hash [Hash] Hash contents of full product metadata
    # @return [Hash] product metadata with product version value updated
    def update_metadata(metadata_hash)
      metadata_hash[PRODUCT_VERSION_KEY].gsub!(/^.*$/, version)
      metadata_hash
    end

    # @param migrations_hash [Hash] Hash contents of full content migrations
    # @return [Hash] content migrations with product version value updated if necessary
    def update_content_migrations(migrations_hash)
      migrations_hash[TO_VERSION_KEY] = version unless migrations_hash[TO_VERSION_KEY].nil?
      migrations_hash.fetch(MIGRATIONS_KEY, []).each do |migration|
        migration.fetch(MIGRATIONS_RULES_KEY, []).each do |rule|
          next unless updates_product_version?(rule)
          rule[MIGRATIONS_RULES_TO_KEY] = version
        end
      end
      migrations_hash
    end

    private

    attr_reader :product_directory_path, :version

    def updates_product_version?(rule_hash)
      rule_hash['type'] == 'update' && rule_hash['selector'] == 'product_version'
    end
  end
end

# Copyright (c) 2014-2015 Pivotal Software, Inc.
# All rights reserved.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
# USE OR OTHER DEALINGS IN THE SOFTWARE.
